@@ -43,12 +43,18 @@ constructor pick {} {
 			$w.mbar.apple add command \
 				-label [mc "About %s" [appname]] \
 				-command do_about
+			$w.mbar.apple add command \
+				-label [mc "Show SSH Key"] \
+				-command do_ssh_key
 		} else {
 			$w.mbar add cascade -label [mc Help] -menu $w.mbar.help
 			menu $w.mbar.help
 			$w.mbar.help add command \
 				-label [mc "About %s" [appname]] \
 				-command do_about
+			$w.mbar.help add command \
+				-label [mc "Show SSH Key"] \
+				-command do_ssh_key
 		}
 
 		wm protocol $top WM_DELETE_WINDOW exit
@@ -381,16 +387,15 @@ method _do_new {} {
 	label $w_body.where.l -text [mc "Directory:"]
 	entry $w_body.where.t \
 		-textvariable @local_path \
-		-font font_diff \
+		-borderwidth 1 \
+		-relief sunken \
 		-width 50
 	button $w_body.where.b \
 		-text [mc "Browse"] \
 		-command [cb _new_local_path]
 	set w_localpath $w_body.where.t
 
-	pack $w_body.where.b -side right
-	pack $w_body.where.l -side left
-	pack $w_body.where.t -fill x
+	grid $w_body.where.l $w_body.where.t $w_body.where.b -sticky ew
 	pack $w_body.where -fill x
 
 	trace add variable @local_path write [cb _write_local_path]
@@ -465,20 +470,22 @@ method _do_clone {} {
 	frame $w_body.args
 	pack $args -fill both
 
-	label $args.origin_l -text [mc "URL:"]
+	label $args.origin_l -text [mc "Source Location:"]
 	entry $args.origin_t \
 		-textvariable @origin_url \
-		-font font_diff \
+		-borderwidth 1 \
+		-relief sunken \
 		-width 50
 	button $args.origin_b \
 		-text [mc "Browse"] \
 		-command [cb _open_origin]
 	grid $args.origin_l $args.origin_t $args.origin_b -sticky ew
 
-	label $args.where_l -text [mc "Directory:"]
+	label $args.where_l -text [mc "Target Directory:"]
 	entry $args.where_t \
 		-textvariable @local_path \
-		-font font_diff \
+		-borderwidth 1 \
+		-relief sunken \
 		-width 50
 	button $args.where_b \
 		-text [mc "Browse"] \
@@ -981,15 +988,14 @@ method _do_open {} {
 	label $w_body.where.l -text [mc "Repository:"]
 	entry $w_body.where.t \
 		-textvariable @local_path \
-		-font font_diff \
+		-borderwidth 1 \
+		-relief sunken \
 		-width 50
 	button $w_body.where.b \
 		-text [mc "Browse"] \
 		-command [cb _open_local_path]
 
-	pack $w_body.where.b -side right
-	pack $w_body.where.l -side left
-	pack $w_body.where.t -fill x
+	grid $w_body.where.l $w_body.where.t $w_body.where.b -sticky ew
 	pack $w_body.where -fill x
 
 	trace add variable @local_path write [cb _write_local_path]
