@@ -1,7 +1,7 @@
 #!/bin/sh
 #
 # Copyright (c) 2006 Eric Wong
-test_description='git-svn commit-diff'
+test_description='git svn commit-diff'
 . ./lib-git-svn.sh
 
 test_expect_success 'initialize repo' '
@@ -26,16 +26,16 @@ prev=`git rev-parse --verify HEAD^1`
 
 test_expect_success 'test the commit-diff command' '
 	test -n "$prev" && test -n "$head" &&
-	git-svn commit-diff -r1 "$prev" "$head" "$svnrepo" &&
+	git svn commit-diff -r1 "$prev" "$head" "$svnrepo" &&
 	svn co "$svnrepo" wc &&
 	cmp readme wc/readme
 	'
 
-test_expect_success 'commit-diff to a sub-directory (with git-svn config)' '
+test_expect_success 'commit-diff to a sub-directory (with git svn config)' '
 	svn import -m "sub-directory" import "$svnrepo"/subdir &&
-	git-svn init --minimize-url "$svnrepo"/subdir &&
-	git-svn fetch &&
-	git-svn commit-diff -r3 "$prev" "$head" &&
+	git svn init --minimize-url "$svnrepo"/subdir &&
+	git svn fetch &&
+	git svn commit-diff -r3 "$prev" "$head" &&
 	svn cat "$svnrepo"/subdir/readme > readme.2 &&
 	cmp readme readme.2
 	'
